#define FASTLED_INTERNAL
#include "FastLED.h"

#include <Time.h>
#include <TimeLib.h>
#include "DS3231.h"
#include <Wire.h>

// Number of RGB LEDs in the strand
#define NUM_LEDS1 6
#define NUM_LEDS2 6
#define NUM_LEDS3 3
#define NUM_LEDS4 9

// Define the array of leds
CRGB leds1[NUM_LEDS1];
CRGB leds2[NUM_LEDS2];
CRGB leds3[NUM_LEDS3];
CRGB leds4[NUM_LEDS4];
// Arduino pins used for StripLeds
#define PIN_LEDS1 2
#define PIN_LEDS2 3 
#define PIN_LEDS3 4
#define PIN_LEDS4 5
// Arduino pins used for buttons
#define PIN_MODE  6
#define PIN_PLUS  7
#define PIN_MINUS 8

int intensity_leds1_2 = 50;
int intensity_leds3 = 100;
int intensity_leds4 = 100;

void StripLedsClock (int type_time, int value, int on_off) {
  // Switch on or off the leds in the boxes belonging to the value
  // type_time = 1 -> value = hour
  // type_time = 2 -> value = minute
  // type_time = 3 -> value = secons
  // on_off = 1 -> on
  // on_off = 0 -> off
  int count_fibonacci;
  int fibonacci_1, fibonacci_2, fibonacci_3, fibonacci_5;
  int pm_am, fibonacci_12, fibonacci_24, fibonacci_36, fibonacci_48;
  
  count_fibonacci = value;

  // Switch off the strip leds in the circles
  fibonacci_12 = 0;
  fibonacci_24 = 0;
  fibonacci_36 = 0;
  fibonacci_48 = 0;
  pm_am = 0;
  switch (type_time) {
    case 1:           // hour
      leds4[8].r = pm_am;
      break;
    case 2:         // minute
      leds4[6].g = fibonacci_12;
      leds4[7].g = fibonacci_24;
      leds4[5].g = fibonacci_36;
      leds4[4].g = fibonacci_48;
      break;
    case 3:         // second
      leds4[3].b = fibonacci_12;
      leds4[1].b = fibonacci_24;
      leds4[2].b = fibonacci_36;
      leds4[0].b = fibonacci_48;
      break;
  }
  
  // Leds in the circles: value greater than 11
  switch (type_time) {
    case 1:           // hour
      if ((value >= 12) and (value < 24)) {
        count_fibonacci = value - 12;
        pm_am = intensity_leds4;
      } else {
        if (value >= 24) count_fibonacci = 0;
      }
      break;
    case 2:           // minute
      if ((value >= 12) and (value < 24)) {
        count_fibonacci = value - 12;
        fibonacci_12 = intensity_leds4;
      }
      if ((value >= 24) and (value < 36)) {
        count_fibonacci = value - 24;
        fibonacci_24 = intensity_leds4;
      }
      if ((value >= 36) and (value < 48)) {
        count_fibonacci = value - 36;
        fibonacci_36 = intensity_leds4;
      }
      if (value >= 48) {
        count_fibonacci = value - 48;
        fibonacci_48 = intensity_leds4;  
      }
      break;
    case 3:           // second
      if ((value >= 12) and (value < 24)) {
        count_fibonacci = value - 12;
        fibonacci_12 = intensity_leds4; 
      }
      if ((value >= 24) and (value < 36)) {
        count_fibonacci = value - 24;
        fibonacci_24 = intensity_leds4;
      }
      if ((value >= 36) and (value < 48)) {
        count_fibonacci = value - 36;
        fibonacci_36 = intensity_leds4;
      }
      if (value >= 48) {
        count_fibonacci = value - 48;
        fibonacci_48 = intensity_leds4;
      }
      break;
  }
  if (on_off == 0) {
    fibonacci_12 = 0;
    fibonacci_24 = 0;
    fibonacci_36 = 0;
    fibonacci_48 = 0;
    pm_am = 0;
  }
  
  switch (type_time) {
    case 1:           // hour
      leds4[8].r = pm_am;
      break;
    case 2:         // minute
      leds4[6].g = fibonacci_12;
      leds4[7].g = fibonacci_24;
      leds4[5].g = fibonacci_36;
      leds4[4].g = fibonacci_48;
      break;
    case 3:         // second
      leds4[3].b = fibonacci_12;
      leds4[1].b = fibonacci_24;
      leds4[2].b = fibonacci_36;
      leds4[0].b = fibonacci_48;
      break;
  }

  // Switch off the strip leds in the boxes
  fibonacci_1 = 0;
  fibonacci_2 = 0;
  fibonacci_3 = 0;
  fibonacci_5 = 0;
  switch (type_time) {
    case 1:           // hour
      leds2[5].r = fibonacci_1;
      leds1[5].r = fibonacci_2;
      leds1[4].r = fibonacci_3;
      leds3[2].r = fibonacci_5;
      break;
    case 2:         // minute
      leds2[3].g = fibonacci_1;
      leds1[3].g = fibonacci_2;
      leds1[2].g = fibonacci_3;
      leds3[1].g = fibonacci_5;
      break;
    case 3:         // second
      leds2[1].b = fibonacci_1;
      leds1[1].b = fibonacci_2;
      leds1[0].b = fibonacci_3;
      leds3[0].b = fibonacci_5;
      break;
  }

  // Leds for the fibonacci basic count from 1 to 11
  switch (count_fibonacci) {
    case 1:
      fibonacci_1 = intensity_leds1_2;
      break;
    case 2:
      fibonacci_2 = intensity_leds1_2;
      break;
    case 3:
      fibonacci_3 = intensity_leds1_2;
      break;
    case 4:
      fibonacci_1 = intensity_leds1_2;
      fibonacci_3 = intensity_leds1_2;
      break;
    case 5:
      fibonacci_5 = intensity_leds3;
      break;
    case 6:
      fibonacci_5 = intensity_leds3;
      fibonacci_1 = intensity_leds1_2;
      break;
    case 7:
      fibonacci_5 = intensity_leds3;
      fibonacci_2 = intensity_leds1_2;
      break;
    case 8:
      fibonacci_5 = intensity_leds3;
      fibonacci_3 = intensity_leds1_2;
      break;
    case 9:
      fibonacci_5 = intensity_leds3;
      fibonacci_3 = intensity_leds1_2;
      fibonacci_1 = intensity_leds1_2;
      break;
    case 10:
      fibonacci_5 = intensity_leds3;
      fibonacci_3 = intensity_leds1_2;
      fibonacci_2 = intensity_leds1_2;
      break;
    case 11:
      fibonacci_5 = intensity_leds3;
      fibonacci_3 = intensity_leds1_2;
      fibonacci_2 = intensity_leds1_2;
      fibonacci_1 = intensity_leds1_2;
      break;
  }

  if (on_off == 0) {
    fibonacci_1 = 0;
    fibonacci_2 = 0;
    fibonacci_3 = 0;
    fibonacci_5 = 0;    
  }
  
  switch (type_time) {
    case 1:           // hour
      leds2[5].r = fibonacci_1;
      leds1[5].r = fibonacci_2;
      leds1[4].r = fibonacci_3;
      leds3[2].r = fibonacci_5;
      break;
    case 2:         // minute
      leds2[3].g = fibonacci_1;
      leds1[3].g = fibonacci_2;
      leds1[2].g = fibonacci_3;
      leds3[1].g = fibonacci_5;
      break;
    case 3:         // second
      leds2[1].b = fibonacci_1;
      leds1[1].b = fibonacci_2;
      leds1[0].b = fibonacci_3;
      leds3[0].b = fibonacci_5;
      break;
  }
  FastLED.show();
}

int counter = 0;
RTClib RTC;
DS3231 Clock;
int setMode = 0;
unsigned long current_hour, current_minute, current_second, last_check, timenow;
int delay_time = 150;
DateTime current_date;

void setup()
{
  Wire.begin();
  
  pinMode(PIN_MODE,  INPUT_PULLUP);
  pinMode(PIN_PLUS,  INPUT_PULLUP);
  pinMode(PIN_MINUS, INPUT_PULLUP);
  
  FastLED.addLeds<NEOPIXEL, PIN_LEDS1>(leds1, NUM_LEDS1);
  FastLED.addLeds<NEOPIXEL, PIN_LEDS2>(leds2, NUM_LEDS2);
  FastLED.addLeds<NEOPIXEL, PIN_LEDS3>(leds3, NUM_LEDS3);
  FastLED.addLeds<NEOPIXEL, PIN_LEDS4>(leds4, NUM_LEDS4);
  
}

void loop() {
  timenow = millis();
  current_date = RTC.now();
  current_hour = current_date.hour();
  current_minute = current_date.minute();
  current_second = current_date.second();

  if (digitalRead(PIN_MODE) == LOW) { // Button mode pressed
    switch (setMode) {
      case 0:     // No mode
        setMode = 1;
        break;
      case 1:     // Set Hour
        setMode = 2;
        break;
      case 2:     // Set Minute
        setMode = 1;
        break;
    }
    last_check = timenow;
  }
  if (setMode == 1) {
    if (timenow-last_check > 10000) {
      setMode = 0;
    } else {
      StripLedsClock(2,current_minute,1);
      if (digitalRead(PIN_PLUS) == LOW) { // Button + pressed
        current_hour++;
        if (current_hour > 23) current_hour = 1;
        last_check = millis();
      }
      if (digitalRead(PIN_MINUS) == LOW) { // Button - pressed
        current_hour--;
        if (current_hour < 1) current_hour = 23;
        last_check = millis();
      }
      StripLedsClock(1,current_hour,1);
      delay(delay_time);
      StripLedsClock(1,current_hour,0);
      // Update the DS3231 time
      current_second = 0;
      Clock.setHour(current_hour);
		  Clock.setMinute(current_minute);
		  Clock.setSecond(0);
    }
  }
  if (setMode == 2) {    
    if (timenow-last_check > 10000) {
      setMode = 0;
    } else {
      StripLedsClock(1,current_hour,1);
      if (digitalRead(PIN_PLUS) == LOW) { // Button + pressed
        current_minute++;
        if (current_minute > 59) current_minute = 1;
        last_check = millis();
      }
      if (digitalRead(PIN_MINUS) == LOW) { // Button - pressed
        current_minute--;
        if (current_minute < 1) current_minute = 59;
        last_check = millis();
      }
      StripLedsClock(2,current_minute,1);
      delay(delay_time);
      StripLedsClock(2,current_minute,0);
      // Update the DS3231 time
      current_second = 0;
      Clock.setHour(current_hour);
		  Clock.setMinute(current_minute);
		  Clock.setSecond(0);
    }
  }
  if (setMode == 0) {
    StripLedsClock(1,current_hour,1);
    StripLedsClock(2,current_minute,1);
    StripLedsClock(3,current_second,1);
  }
  delay(delay_time);
}
